package com.chinaitop.depot.device.controller;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.chinaitop.depot.device.model.BasicEquipmentEquipmentPool;
import com.chinaitop.depot.device.model.BasicEquipmentEquipmentPoolExample;
import com.chinaitop.depot.device.service.BasicEquipmentEquipmentPoolService;
import com.chinaitop.depot.device.service.impl.ImportServiceImpl;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 设备器材库Controller
 * @author User
 *
 */
@RestController
@RequestMapping(value="/BasicEquipmentEquipmentPool")
@Api(value= "BasicEquipmentEquipmentPoolController", description = "设备器材库")
public class BasicEquipmentEquipmentPoolController {

	@Resource
	private BasicEquipmentEquipmentPoolService poolServivce;
	@Resource
	private ImportServiceImpl importService;

	/**
	 * 查询数据列表
	 * 
	 * @param request request对象
	 * @param pageNum 页数
	 * @param pageSize 每页显示的条数
	 * @param shelfId 货架号
	 * @param warehouseId 货位号
	 * @return
	 */
	@RequestMapping(value = "/getList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="查询数据信息", notes = "查询数据列表，支持分页")
  	@ApiImplicitParams({
  			@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
  			@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
  			@ApiImplicitParam(name = "orgId", value = "当前登录人组织id", paramType = "query")
  	})
	public PageInfo<BasicEquipmentEquipmentPool> getList(HttpServletRequest request, Integer pageNum, Integer pageSize,
			String shelfId, String warehouseId ,Integer orgId) {
		//Integer depotId = (Integer) request.getSession().getAttribute("depotId");
		BasicEquipmentEquipmentPoolExample example = new BasicEquipmentEquipmentPoolExample();
		BasicEquipmentEquipmentPoolExample.Criteria criteria = example.createCriteria();

		if (null != shelfId && !"".equals(shelfId)) {
			criteria.andShelfIdLike("%"+shelfId+"%");
		}

		if (null != warehouseId && !"".equals(warehouseId)) {
			criteria.andWarehouseIdLike("%"+warehouseId+"%");
		}
		

		criteria.andOrgIdEqualTo(orgId); //只能查询当前单位的数据
		example.setOrderByClause("id asc");
		
		if (pageNum != null && pageSize != null) {
			PageHelper.startPage(pageNum, pageSize);
		}
		List<BasicEquipmentEquipmentPool> list = poolServivce.findByCondition(example);

		PageInfo<BasicEquipmentEquipmentPool> pageInfo = new PageInfo<BasicEquipmentEquipmentPool>(list);

		return pageInfo;
	}

	/**
	 * 查询一条数据
	 * 
	 * @param id 数据ID
	 * @return
	 */
	@RequestMapping(value ="/edit",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="根据id查询数据", notes = "根据id查询数据")
	@ApiImplicitParams({
		  @ApiImplicitParam(name = "id", value = "id", paramType = "query")
	})
	public BasicEquipmentEquipmentPool edit(Integer id) {
		BasicEquipmentEquipmentPool pool = null;
		if (id != null) {
			pool = poolServivce.findByIdObj(id);
		}
		return pool;
	}

	/**
	 * 更新数据
	 * 
	 * @param request request对象
	 * @param poolJson 数据对象
	 * @return
	 */
	@RequestMapping(value="/update", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="更新数据", notes = "修改数据")
	@ApiImplicitParams({
		  @ApiImplicitParam(name = "userId", value = "当前登录人id", paramType = "form"),
		  @ApiImplicitParam(name = "orgId", value = "当前登录人组织id", paramType = "form"),
		  @ApiImplicitParam(name = "poolJson", value = "数据对象", paramType = "form")
	})
	public Map<String, Object> update(String poolJson ,Integer userId ,Integer orgId) {
		Map<String, Object> map = new HashMap<String, Object>();
		ObjectMapper mapper = new ObjectMapper();
		try {
			BasicEquipmentEquipmentPool pool = mapper.readValue(poolJson, BasicEquipmentEquipmentPool.class);
			if (pool.getId() == null) {
				pool.setOrgId(orgId);
				pool.setInputPersonId(userId);
				pool.setInputDate(new Date());
				poolServivce.add(pool);
			} else {
				poolServivce.update(pool);
			}
			map.put("status", "success");
			map.put("msg", "保存成功！");
		} catch (Exception e) {
			e.printStackTrace();
		}
		return map;
	}

	/**
	 * 删除
	 * 
	 * @param id 数据ID
	 * @return
	 */
	@RequestMapping(value ="/remove",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="根据id删除数据", notes = "根据id删除数据")
	@ApiImplicitParams({
		  @ApiImplicitParam(name = "id", value = "id", paramType = "form")
	})
	public Map<String, Object> remove(Integer id) {
		Map<String, Object> map = new HashMap<String, Object>();
		poolServivce.remove(id);
		map.put("status", "success");
		map.put("msg", "删除成功！");
		return map;
	}

	/**
     * 导入excel
     * @param file 文件
     * @param fileType 类型：如xlsx
     * @param request
     * @return
     */
    @RequestMapping(value ="/importFile",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="导入Excel", notes = "导入Excel")
    @ApiImplicitParams({
    	@ApiImplicitParam(name = "file", value = "文件", paramType = "form"),
    	@ApiImplicitParam(name = "fileType", value = "文件类型", paramType = "form"),
		@ApiImplicitParam(name = "orgId", value = "当前登录人组织id", paramType = "form")
	})
    public String importFile(MultipartFile file, String fileType, Integer orgId , HttpServletRequest request){

        String[] titleArr = {"存放区域", "货架号", "货位号", "备注"};//第一行的title
        boolean[] emptyArr = {false  ,  false  ,  false , true  ,  true};//指定列可为空，不可为空值为false.

        List<List<String>> list = new ArrayList<List<String>>();//该list为数据返回的list
        String errormsg = null;
        try {
        	errormsg = importService.importFile(file, fileType, request, titleArr, emptyArr, list);
        	
        	if (errormsg != null && !"".equals(errormsg)) {
        		return errormsg;
        	}

            if (list.size()>0) {
            	errormsg = poolServivce.importData(list, orgId);

            	System.out.println(list);
            	System.out.println(errormsg);
            	//errormsg = "导入成功！";
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return errormsg;
    }
}
