package com.chinaitop.depot.device.controller;

import com.chinaitop.depot.device.model.BasicEquipmentEquipmentPool;
import com.chinaitop.depot.device.service.BasicEquipmentEquipmentPoolService;
import com.chinaitop.depot.device.model.StorageDevicecheck;
import com.chinaitop.depot.device.model.StorageDevicecheckExample;
import com.chinaitop.depot.device.model.StorageDeviceinput;
import com.chinaitop.depot.device.model.StorageDeviceinputExample;
import com.chinaitop.depot.device.service.StorageDeviceCheckService;
import com.chinaitop.depot.utils.ParameterUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(value= {"/deviceCheck"})
@Api(value= "StorageDeviceCheckController", description = "器材设备盘点")
public class StorageDeviceCheckController {

	@Resource
	private StorageDeviceCheckService storageDeviceCheckService;
	@Resource
	private BasicEquipmentEquipmentPoolService equipmentPoolService;
	
	 /**
     * 设备器材管理-盘点单查询
	 * @throws Exception 
     */
    @RequestMapping(value = "/getCheckList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="盘点单查询", notes = "设备器材管理-盘点单查询，支持分页")
  	@ApiImplicitParams({
  			@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
  			@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query")
  	})
    public PageInfo<StorageDevicecheck> listCheckPage(Integer pageNum, Integer pageSize, String pdNum,String starttime,String endtime,Integer orgId) throws Exception{

        StorageDevicecheckExample storageDevicecheckExample=new StorageDevicecheckExample();
        StorageDevicecheckExample.Criteria criteria= storageDevicecheckExample.createCriteria();
        if(ParameterUtil.isnotnull(pdNum)) criteria.andCheckNumberLike("%"+pdNum+"%");
       if(ParameterUtil.isnotnull(starttime)) criteria.andCheckDateGreaterThanOrEqualTo(ParameterUtil.string2datetime(starttime+" 00:00:00"));
       if(ParameterUtil.isnotnull(endtime)) criteria.andCheckDateLessThanOrEqualTo(ParameterUtil.string2datetime(endtime+" 00:00:00"));
        if(ParameterUtil.isnotnull(orgId)){
            criteria.andOrgIdEqualTo(orgId);
        }
        //按照id进行降序
        storageDevicecheckExample.setOrderByClause("id desc");
        if (pageNum!=null && pageSize!=null) PageHelper.startPage(pageNum, pageSize);

        List<StorageDevicecheck> list = storageDeviceCheckService.queryByExample(storageDevicecheckExample);
        if(list!=null && list.size()>0){
        	for (StorageDevicecheck storageDevicecheck : list) {
        		if(storageDevicecheck.getDepotId()!=null){
        			BasicEquipmentEquipmentPool equipmentPool=equipmentPoolService.findByIdObj(storageDevicecheck.getDepotId());
        			if(equipmentPool != null) storageDevicecheck.setDepotName(equipmentPool.getPutPosition());
        		}
        	}
        }

        PageInfo<StorageDevicecheck> pageInfo = new PageInfo<StorageDevicecheck>(list);
        return pageInfo;
    }
    /**
     * 设备器材管理-盘点单查询 NoPage
	 * @throws Exception
     */
    @RequestMapping(value = "/getCheckListNoPage",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="盘点单查询", notes = "设备器材管理-盘点单查询，不支持分页")
    public List<StorageDevicecheck> listCheckPageNoPage() throws Exception{

        StorageDevicecheckExample storageDevicecheckExample=new StorageDevicecheckExample();
        storageDevicecheckExample.setOrderByClause("check_number desc");

        List<StorageDevicecheck> list = storageDeviceCheckService.queryByExample(storageDevicecheckExample);

        return list;
    }


    /**
     * 根据盘点种类获取子类数据
     * @return
     */
    @RequestMapping(value = "/childZlList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="根据盘点种类获取子类数据", notes = "根据盘点种类获取子类数据")
  	@ApiImplicitParams({
  			@ApiImplicitParam(name = "deviceType", value = "类型", paramType = "query"),
  			@ApiImplicitParam(name = "position", value = "器材的存放位置", paramType = "query")
  	})
    public List<StorageDeviceinput> childZlList(Integer position,String deviceType){
        List<StorageDeviceinput> list = null;
        StorageDeviceinputExample storageDeviceinputExample=new StorageDeviceinputExample();
        StorageDeviceinputExample.Criteria criteria=storageDeviceinputExample.createCriteria();
        if(ParameterUtil.isnotnull(deviceType)){
            criteria.andDeviceTypeEqualTo(Integer.valueOf(deviceType));
        }
        if(ParameterUtil.isnotnull(position)){
            criteria.andPositionEqualTo(Integer.toString(position));
        }
        try {
            list = storageDeviceCheckService.childZlList(storageDeviceinputExample);
            /*for (StorageDeviceinput storageDevicecheck : list) {
                if(storageDevicecheck.getPosition()!=null){
                	BasicEquipmentEquipmentPool equipmentPool=equipmentPoolService.findByIdObj(position);
                    if(equipmentPool != null) storageDevicecheck.(equipmentPool.getPutPosition());

                }
            }*/
        } catch (Exception e) {
            e.printStackTrace();
        }
        return list;
    }
    /**
     * 根据盘点种类获取子类数据
     * @return
     */
    @RequestMapping(value = "/getCountForDate",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="根据盘点种类获取子类数据", notes = "根据盘点种类获取子类数据")
  	@ApiImplicitParams({
  			@ApiImplicitParam(name = "depotId", value = "depotId", paramType = "query")
  	})
    public int getCountForDate(String depotId){
        Map<String,Object> map = new HashMap<>();
        map.put("checkDate","\""+ParameterUtil.getSysDate()+"%\"");
        map.put("depotId",depotId);
        int count  = storageDeviceCheckService.getCountForDate(map);
        return count;
    }

    /**
     * 根据depotid删除数据
     */
    @RequestMapping(value="/remove", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="删除数据", notes = "根据条件删除数据")
	@ApiImplicitParams({
	   	 @ApiImplicitParam(name = "depotid", value = "depotid", paramType = "form"),
	   	 @ApiImplicitParam(name = "checkNum", value = "checkNum", paramType = "form")
	}) 
    public Map<String, Object> remove(Integer depotId,String checkNum){
        Map<String,Object> retMap = new HashMap<String,Object>();
        storageDeviceCheckService.remove(depotId,checkNum);
        retMap.put("msg","success");
        return retMap;
    }

    /**
     * 新增
     */
    @RequestMapping(value="/save", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="保存数据", notes = "保存增加或修改的数据")
    @ApiImplicitParams({
	   	 @ApiImplicitParam(name = "deviceCheckJson", value = "新增数据", paramType = "form"),
	   	 @ApiImplicitParam(name = "type", value = "类型", paramType = "form")
	})
    public Map<String, Object>  add(String deviceCheckJson,String type){
        Map<String, Object> modelMap = new HashMap<>();
        try {
            storageDeviceCheckService.add(deviceCheckJson,type);
            modelMap.put("status", "success");
        } catch (Exception e) {
            e.printStackTrace();
            modelMap.put("status", "error");
        }
        return modelMap;
    }

      /**
     * 根据id加载对应数据
     * @param id
     * @return
     */
    @RequestMapping(value ="/loadDataByDepotId",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="查询数据", notes = "根据条件查询数据")
	@ApiImplicitParams({
		 @ApiImplicitParam(name = "id", value = "id", paramType = "query"),
		 @ApiImplicitParam(name = "pddh", value = "盘点单号", paramType = "query")
	})
    public List<StorageDevicecheck> loadDataByDepotId(Integer id,String pddh){
    	List<StorageDevicecheck> storageDevicekeepList =storageDeviceCheckService.loadDataByDepotId(id,pddh);
        for (StorageDevicecheck storageDeviceCheck : storageDevicekeepList) {
           
                if(storageDeviceCheck.getDepotId()!=null){
                   /* BasicStorehouse storehouse=storehouseService.findByPrimary(storageDeviceCheck.getDepotId());
                    if(storehouse != null) storageDeviceCheck.setDepotName(storehouse.getStorehouseName());*/
                	BasicEquipmentEquipmentPool equipmentPool=equipmentPoolService.findByIdObj(storageDeviceCheck.getDepotId());
                    if(equipmentPool != null) storageDeviceCheck.setDepotName(equipmentPool.getPutPosition());

                }
            } 
       
        return storageDevicekeepList;
    }
}
