package com.chinaitop.depot.device.controller;

import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.device.model.StorageDeviceinput;
import com.chinaitop.depot.device.model.StorageDeviceinputExample;
import com.chinaitop.depot.device.model.StorageDevicekeep;
import com.chinaitop.depot.device.model.StorageDevicekeepExample;
import com.chinaitop.depot.device.service.StorageDeviceInputService;
import com.chinaitop.depot.device.service.StorageDeviceKeepService;
import com.chinaitop.depot.utils.ParameterUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping(value= {"/deviceKeep"})
@Api(value= "StorageDeviceKeepController", description = "器材设备保养")
public class StorageDeviceKeepController {

	@Resource
	private StorageDeviceKeepService deviceKeepService;
	@Resource
	private StorageDeviceInputService deviceInputService;
	/*@Resource
	private OrgInfoService orgInfoService;*/
	
	 /**
     * 分页获取器材信息
	 * @throws Exception 
     */
    @RequestMapping(value = "/getDeviceList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="查询保养信息", notes = "查询保养信息，支持分页")
  	@ApiImplicitParams({
  			@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
  			@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "deviceName", value = "设备名称", paramType = "query")
  	})
    public PageInfo<StorageDevicekeep> listCheckPage(HttpServletRequest request,Integer pageNum, Integer pageSize, String deviceName, 
    		Integer keepStatus, Integer orgId) throws Exception{

    	StorageDevicekeepExample storageDevicekeepExample=new StorageDevicekeepExample();
    	StorageDevicekeepExample.Criteria criteria=storageDevicekeepExample.createCriteria();
       
    	if(ParameterUtil.isnotnull(deviceName)){
            criteria.andDeviceNameLike(deviceName);
        }
        if(ParameterUtil.isnotnull(keepStatus)){
           criteria.andKeepStatusEqualTo(keepStatus);
        }
        if(ParameterUtil.isnotnull(orgId)){
        	criteria.andOrgIdEqualTo(orgId);
        }
    	

        //排序
        storageDevicekeepExample.setOrderByClause("FIELD(keep_status,1,0), keep_end desc , id desc");
        
   
        if (pageNum!=null && pageSize!=null) {
            PageHelper.startPage(pageNum, pageSize);
        }
        List<StorageDevicekeep> list = deviceKeepService.queryByDevice(storageDevicekeepExample);
        PageInfo<StorageDevicekeep> pageInfo = new PageInfo<StorageDevicekeep>(list);
        return pageInfo;
    }

    /**
     * 根据主键id删除数据
     */
    @RequestMapping(value="/remove", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="删除数据", notes = "根据id删除数据")
	@ApiImplicitParams({
	   	@ApiImplicitParam(name = "id", value = "id", paramType = "form"),
	}) 
    public Map<String, Object> remove(Integer id){
        Map<String,Object> retMap = new HashMap<String,Object>();
        deviceKeepService.remove(id);
        retMap.put("msg","success");
        return retMap;
    }
	
    /**
     * 结束保养
     */
    @RequestMapping(value="/end", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="结束保养", notes = "结束保养")
    public Map<String, Object> end(String deviceListJson) {
        Map<String, Object> map = new HashMap<>();
        // JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        StorageDevicekeep storageDevicekeep = null;
        try {
        	storageDevicekeep = (StorageDevicekeep)mapper.readValue(deviceListJson, StorageDevicekeep.class);
        	storageDevicekeep.setKeepEnd(new Date());
        	storageDevicekeep.setKeepStatus(0);
        	deviceKeepService.updateByPrimaryKeySelective(storageDevicekeep);
        	//根据id获取编号和型号
        	storageDevicekeep=deviceKeepService.selectById(storageDevicekeep.getId());
        	//改变入库表的状态
        	map.put("number", storageDevicekeep.getNumber());
        	map.put("model", storageDevicekeep.getModel());
        	map.put("deviceNameId", storageDevicekeep.getDeviceName());
        	map.put("orgId", storageDevicekeep.getOrgId());
        	List<StorageDeviceinput> inputList=deviceInputService.getUseCount(map);
        	inputList.get(0).setStatus(0);//正常状态
        	inputList.get(0).setUseCount(inputList.get(0).getUseCount()+1);//数量增加
        	deviceInputService.updateByPrimaryKeySelective(inputList.get(0));
        	deviceKeepService.updateByPrimaryKeySelective(storageDevicekeep);
            map.put("status", "success");
        } catch (IOException e) {
            map.put("status", "error");
            e.printStackTrace();
        }
        return map;
    }
    
    /**
     * 批量结束保养
     */
    @RequestMapping(value="/endMore", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="批量结束保养", notes = "批量结束保养")
    public Map<String, Object> endMore(String deviceListJson) {
        Map<String, Object> modelMap = new HashMap<>();
        // JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        try {
        	JSONObject jsonObject = JSONObject.parseObject(deviceListJson);
        	String idStr = jsonObject.getString("id");
        	
        	if(idStr != null && !"".equals(idStr)){
        		String[] ids = idStr.split(",");
        		for(int i=0; i<ids.length; i++){
        			StorageDevicekeep storageDevicekeep = new StorageDevicekeep();
                	storageDevicekeep.setKeepEnd(new Date());
                	storageDevicekeep.setKeepStatus(0);
                	storageDevicekeep.setId(Integer.parseInt(ids[i]));
                	deviceKeepService.updateByPrimaryKeySelective(storageDevicekeep);
        		}
        	}
            modelMap.put("status", "success");
        } catch (Exception e) {
            modelMap.put("status", "error");
            e.printStackTrace();
        }
        return modelMap;
    }


	 /**
     * 新增保养
     */
    @RequestMapping(value="/save", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
   	@ApiOperation(value="新增保养", notes = "新增保养")
    public Map<String, Object> save(String deviceAddJson) {
        Map<String, Object> modelMap = new HashMap<>();
        // JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        StorageDevicekeep storageDevicekeep = null;
        try {
        	storageDevicekeep = (StorageDevicekeep)mapper.readValue(deviceAddJson, StorageDevicekeep.class);
        	storageDevicekeep.setKeepStatus(1);
        	deviceKeepService.save(storageDevicekeep);
        	//根据设备编号获取入库表的改设备信息
        	modelMap.put("number", storageDevicekeep.getNumber());
        	modelMap.put("model", storageDevicekeep.getModel());
        	modelMap.put("deviceNameId", storageDevicekeep.getDeviceName());
        	modelMap.put("orgId", storageDevicekeep.getOrgId());
        	List<StorageDeviceinput> inputList=deviceInputService.getUseCount(modelMap);
        	inputList.get(0).setStatus(2);//保养状态
        	inputList.get(0).setUseCount(inputList.get(0).getUseCount()-1);//数量减少
        	deviceInputService.updateByPrimaryKeySelective(inputList.get(0));
            modelMap.put("status", "success");
        } catch (IOException e) {
            modelMap.put("status", "error");
            e.printStackTrace();
        }
        return modelMap;
    }

    
      /**
     * 根据id加载对应数据
     * @param id
     * @return
     */
    @RequestMapping(value="/loadDataById", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
   	@ApiOperation(value="获取数据", notes = "根据id获取数据")
   	@ApiImplicitParams({
   	   	@ApiImplicitParam(name = "id", value = "id", paramType = "form"),
   	}) 
    public List<StorageDevicekeep> loadDataById(int id){
    	List<StorageDevicekeep> list=new ArrayList<StorageDevicekeep>();
    	StorageDevicekeep storageDevicekeep=deviceKeepService.loadDataById(id);
    	list.add(storageDevicekeep);
        return list;
    }
    
    
    /**
     * 获取机械及器材名称列表
     */
    @RequestMapping(value="/getDeviceNameList",produces = MediaType.APPLICATION_JSON_VALUE,  method=RequestMethod.GET)
   	@ApiOperation(value="获取机械及器材名称列表", notes = "获取机械及器材名称列表")
    public List<StorageDeviceinput> getDeviceNameList(){
        return deviceInputService.getdevieName();
    }
    
    /**
     * 获取机械及器材规格型号与编号
     */
    @RequestMapping(value="/getDeviceModelList",produces = MediaType.APPLICATION_JSON_VALUE,  method=RequestMethod.GET)
   	@ApiOperation(value="获取机械及器材规格型号与编号", notes = "获取机械及器材规格型号与编号")
	@ApiImplicitParams({
   	   	@ApiImplicitParam(name = "deviceName", value = "deviceName", paramType = "form"),
   	}) 
    public List<StorageDeviceinput> getDeviceModelList(String deviceName){
    	return deviceInputService.getDeviceModelList(deviceName);
    }
    
    /**
     * 获取机械及器材规格型号与编号（维修）
     */
    @RequestMapping(value="/getDeviceModelListRepair",produces = MediaType.APPLICATION_JSON_VALUE,  method=RequestMethod.GET)
    @ApiOperation(value="获取机械及器材规格型号与编号", notes = "获取机械及器材规格型号与编号")
    @ApiImplicitParams({
    	@ApiImplicitParam(name = "deviceName", value = "deviceName", paramType = "form"),
    }) 
    public List<StorageDeviceinput> getDeviceModelListRepair(String deviceName){
    	return deviceInputService.getDeviceModelListRepair(deviceName);
    }

}
